package org.codehaus.mojo.chronos.report;

import java.io.IOException;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.maven.reporting.AbstractMavenReport;
import org.apache.maven.reporting.MavenReportException;
import org.codehaus.doxia.sink.Sink;
import org.codehaus.mojo.chronos.common.TestDataDirectory;
import org.codehaus.mojo.chronos.common.model.GCSamples;
import org.codehaus.mojo.chronos.common.model.GroupedResponsetimeSamples;
import org.codehaus.mojo.chronos.report.chart.ChartRenderer;
import org.codehaus.mojo.chronos.report.chart.ChartRendererImpl;
import org.codehaus.mojo.chronos.report.chart.ChartUtil;
import org.codehaus.mojo.chronos.report.chart.ChronosReportPlugin;
import org.codehaus.mojo.chronos.report.chart.GraphGenerator;
import org.xml.sax.SAXException;

public abstract class AbstractReportMojo extends AbstractMavenReport {

	protected void generateReport(Locale locale) throws MavenReportException {
		try {
			final TestDataDirectory testDataDirectory = getTestDataDirectory();
			GroupedResponsetimeSamples jmeterSamples = testDataDirectory.readResponsetimeSamples();
			if (jmeterSamples.size() == 0) {
				throw new MavenReportException("Response time samples not found for " + getDataId());
			}

			getLog().info("  tests: " + jmeterSamples.getSampleGroups().size());
			getLog().info("  jmeter samples: " + jmeterSamples.size());

			// charts
			getLog().info(" generating charts...");

			GCSamples gcSamples = testDataDirectory.readGCSamples();
			List<ChronosReportPlugin> defaultPlugins = ChartUtil.createDefaultPlugins(jmeterSamples, gcSamples);
			GraphGenerator graphGenerator = new GraphGenerator(defaultPlugins, getLog());
			ChartRenderer renderer = new ChartRendererImpl(getOutputDirectory());
			graphGenerator.generateGraphs(renderer, getBundle(locale), getConfig());

			// report
			ReportGenerator reportGenerator = new ReportGenerator(getBundle(locale), getConfig(), graphGenerator);
			getLog().info(" generating report...");
			Sink sink = getSink();
			reportGenerator.doGenerateReport(sink, jmeterSamples);

		} catch (IOException e) {
			throw new MavenReportException("ReportGenerator failed", e);
		} catch (SAXException e) {
			throw new MavenReportException("ReportGenerator failed", e);
		} catch (ParserConfigurationException e) {
			throw new MavenReportException("ReportGenerator failed", e);
		}
	}
	
	protected abstract TestDataDirectory getTestDataDirectory();

	private ResourceBundle getBundle(Locale locale) {
		return Utils.getBundle(locale);
	}

	/**
	 * @return Returns the report.
	 */
	private ReportConfig getConfig() {
		return new ReportConfig() {
			public int getAverageduration() {
				return AbstractReportMojo.this.getReportAverageDuration();
			}

			public String getDescription() {
				return AbstractReportMojo.this.getReportDescription();
			}

			/* Merged from Atlassion */
			public double getHistoryChartUpperBound() {
				return AbstractReportMojo.this.getReportHistoryChartUpperBound();
			}

			public String getId() {
				return AbstractReportMojo.this.getId();
			}

			public String getMetadata() {
				return AbstractReportMojo.this.getMetadata();
			}

			public String getTitle() {
				return AbstractReportMojo.this.getReportTitle();
			}

			public boolean isShowaverage() {
				return AbstractReportMojo.this.isShowAverage();
			}

			public boolean isShowdetails() {
				return AbstractReportMojo.this.isShowDetails();
			}

			public boolean isShowgc() {
				return AbstractReportMojo.this.isShowGC();
			}

			public boolean isShowhistogram() {
				return AbstractReportMojo.this.isShowHistogram();
			}

			public boolean isShowinfotable() {
				return AbstractReportMojo.this.isShowInfotable();
			}

			public boolean isShowpercentile95() {
				return AbstractReportMojo.this.isShowPercentile95();
			}

			public boolean isShowpercentile99() {
				return AbstractReportMojo.this.isShowPercentile99();
			}

			public boolean isShowresponse() {
				return AbstractReportMojo.this.isShowResponse();
			}

			public boolean isShowServerlog() {
				return AbstractReportMojo.this.isShowServerLog();
			}

			public boolean isShowsummary() {
				return AbstractReportMojo.this.isShowSummary();
			}

			public boolean isShowsummarycharts() {
				return AbstractReportMojo.this.isShowsummaryCharts();
			}

			public boolean isShowthroughput() {
				return AbstractReportMojo.this.isShowThroughput();
			}

			public boolean isShowtimeinfo() {
				return AbstractReportMojo.this.isShowTimeinfo();
			}
		};
	}

	/**
	 * @see org.apache.maven.reporting.MavenReport#getOutputName()
	 */
	public String getOutputName() {
		return getConfig().getId();
	}

	protected abstract String getId();

	protected abstract String getDataId();

	protected abstract String getMetadata();

	protected abstract int getReportAverageDuration();

	protected abstract String getReportDescription();

	protected abstract double getReportHistoryChartUpperBound();

	protected abstract String getReportTitle();

	protected abstract boolean isShowAverage();

	protected abstract boolean isShowDetails();

	protected abstract boolean isShowGC();

	protected abstract boolean isShowHistogram();

	protected abstract boolean isShowInfotable();

	protected abstract boolean isShowPercentile95();

	protected abstract boolean isShowPercentile99();

	protected abstract boolean isShowResponse();

	protected abstract boolean isShowServerLog();

	protected abstract boolean isShowSummary();

	protected abstract boolean isShowsummaryCharts();

	protected abstract boolean isShowThroughput();

	protected abstract boolean isShowTimeinfo();
}
